#include <VRS51L3074_SDCC.h>



void initAD(void)
{
int cptr = 0x00;
char readflag = 0x00;

	PERIPHEN1 |= 0xC0; 				//Make sure the SPI Interface is activated
	//--Wait activity stops on the SPI interface (Monitor SPINOCS)
	while(!(SPISTATUS &= 0x08));
	SPICTRL = 0x85;				//SPICLK = /32 (1.25MHz)
								//CS0 Active
								//SPI Mode 0 Phase = 0, POL = 0
								//SPI Master Mode

	SPICONFIG = 0x40;				//SPI Chip select is automatic
								//Clear SPIUNDEFC Flag
								//SPILOAD = 0 -> Manual CS3 behaviour
								//No SPI Interrupt used

	SPISTATUS = 0x00;				//SPI transactions are in MSB First Format

	SPISIZE = 0x17;				        //SPI Transaction Size is 24-bits
	//-Dummy Read the SPI RX buffer to clear the RXAV Flag
	readflag = SPIRXTX0;
}

void initPWM(void)
{
int cptr = 0x00;
char readflag = 0x00;

     PERIPHEN2 |= (1<<1); 		// Enable PWM SFRs (PWMSFREN)
     DEVIOMAP |= (1<<6);                // Enable alternate mapping (we use P5.5)
     //CLEAR All PWM Channels
     PWMCFG = 0x20;
     // Configure the PWM prescaler
     PWMCLKCFG = 0x00;                  // no clock prescaler
     // Configure PWM Polarity
     PWMLDPOL = (1<<5);                 // Set PWM5 in inverted polarity, so PWMMID data equals pulse width
                                        // PWM output = 1 until
                                        // PWMMID Value is reached
     //Configure PWM5 END value = 0x03FF (8bit)
     PWMCFG = 0x5D;                     // Point to PWM5 END MSB
     PWMDATA = 0x00;                    // Set Max Count MSB = 0x00
     PWMCFG = 0x4D;                     // Point to PWM5 END LSB
     PWMDATA = 0xFF;                    // Set Max Count = 0xFF
     //Configure PWM5 MID value for off state
     PWMCFG = 0x55;                     // Point to PWM5 MID MSB
     PWMDATA = 0x00;                    // Set PWM MID MSB = 0x00
     PWMCFG = 0x45;                     // Point to PWM5 MID LSB
     PWMDATA = 0x01;                    // Set PWM MID LSB = 0x00

     PWMCFG &= 0x1F;                    // Clear the PWMWAIT bit to initiate
                                        // the PWMs operation
     PWMEN |= (1<<5);                   // Enable PWM5 (PWM5EN)
}

     
#define START	(1<<2)
#define SINGLE	(1<<1)
#define CHANNEL 3

void main ()
{
unsigned int adcdata;

    initAD();
    initPWM();

    while (1){
	//-Perform the SPI read
	SPIRXTX2 = 0x00;			
	SPIRXTX1 = (CHANNEL<<6);
	SPIRXTX0 = START | SINGLE | (CHANNEL>>2);	//Writing to the SPIRXTX0 will trigger the SPI Transaction


	//Wait for SPIRXAVF being set
	while(!(SPISTATUS & 0x02));


	//Read SPI data and put it in PWMMID
	adcdata= (SPIRXTX1 << 8);
	adcdata|= SPIRXTX0;
	adcdata&= 0x0FFF;				//isolate the 12 lsb of the read value
	adcdata>>=4;      // scale down to 8-bits
	if(adcdata==0)    // correct PWM 'flaw'
	  adcdata++;
	//Configure PWM5 MID value (duty = adcdata)
	PWMCFG = 0x55;                     //Point to PWM5 MID MSB
	PWMDATA = (adcdata>>8);            //Set PWM MID MSB
	PWMCFG = 0x45;                     //Point to PWM5 MID LSB
	PWMDATA = adcdata&0x00FF;          //Set PWM MID LSB
	PWMCFG &= 0x1F;                    //Clear the PWMWAIT bit to initiate
    }
}
